package geniusweb.issuevalue;

import static org.junit.Assert.assertEquals;
import static org.mockito.Mockito.mock;

import java.io.IOException;
import java.math.BigDecimal;
import java.util.Arrays;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import org.junit.Test;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;

import geniusweb.issuevalue.Bid;
import geniusweb.issuevalue.DiscreteValue;
import geniusweb.issuevalue.NumberValue;
import geniusweb.issuevalue.Value;
import tudelft.utilities.junit.GeneralTests;

public class BidTest extends GeneralTests<Bid> {

	private final String asString1 = "{\"issuevalues\":{\"issue3\":9012345678901234567.89,\"issue2\":1,\"issue1\":\"b\"}}";
	private final static String ISSUE1 = "issue1";
	private final static Value VALUE1 = new DiscreteValue("value1");
	private final static String ISSUE2 = "issue2";
	private final static Value VALUE2 = new NumberValue("10");
	private final static List<List<Bid>> list = new LinkedList<>();

	private final static Bid bid, bid1, bidb, bidc;
	static {
		Map<String, Value> issuevalues = new HashMap<>();
		issuevalues.put(ISSUE1, new DiscreteValue("b"));
		issuevalues.put(ISSUE2, new NumberValue(BigDecimal.ONE));
		issuevalues.put("issue3",
				new NumberValue(new BigDecimal("9012345678901234567.89")));
		bid = new Bid(issuevalues);

		// different order but that shouldn't matter
		Map<String, Value> issuevalues1 = new HashMap<>();
		issuevalues1.put("issue3",
				new NumberValue(new BigDecimal("9012345678901234567.89")));
		issuevalues1.put(ISSUE2, new NumberValue(BigDecimal.ONE));
		issuevalues1.put(ISSUE1, new DiscreteValue("b"));
		bid1 = new Bid(issuevalues1);

		Map<String, Value> issuevaluesb = new HashMap<>();
		issuevaluesb.put(ISSUE1, new DiscreteValue("b"));
		issuevaluesb.put(ISSUE2, new NumberValue(BigDecimal.ONE));
		bidb = new Bid(issuevaluesb);

		// bidc and bidd have values swapped, so different issuevalues.
		Map<String, Value> issuevaluesc = new HashMap<>();
		issuevaluesc.put(ISSUE1, VALUE1);
		issuevaluesc.put(ISSUE2, VALUE2);
		bidc = new Bid(issuevaluesc);

		list.add(Arrays.asList(bid, bid1));
		list.add(Arrays.asList(bidb));
		list.add(Arrays.asList(bidc));
	}

	@Override
	public List<List<Bid>> getGeneralTestData() {

		return list;
	}

	@Override
	public List<String> getGeneralTestStrings() {
		return Arrays.asList(
				"Bid\\{issue3=9012345678901234567.89, issue2=1, issue1=\"b\"\\}",
				"Bid\\{issue2=1, issue1=\"b\"\\}",
				"Bid\\{issue2=10, issue1=\"value1\"\\}"

		);
	}

	@Test(expected = NullPointerException.class)
	public void testNullConstructor() {
		Map<String, Value> issuevalues = null;
		Bid bid = new Bid(issuevalues);
	}

	@Test
	public void bidTestSimple() {
		Map<String, Value> issuevalues = new HashMap<>();
		new Bid(issuevalues); // no issues, no values should be ok
	}

	@Test(expected = IllegalArgumentException.class)
	public void bidTestNull() {
		Map<String, Value> issuevalues = new HashMap<>();
		issuevalues.put(ISSUE1, null);
		new Bid(issuevalues); // smokes
	}

	@Test
	public void bidTestOkIssueValue() {
		Map<String, Value> issuevalues = new HashMap<>();
		issuevalues.put(ISSUE1, mock(Value.class));

		new Bid(issuevalues); // shouldn't smoke
	}

	@Test
	public void testSerialize() throws JsonProcessingException {

		ObjectMapper jackson = new ObjectMapper();
		System.out.println(jackson.writeValueAsString(bid));
		assertEquals(asString1, jackson.writeValueAsString(bid));

	}

	@Test
	public void testDeserialize() throws IOException {

		ObjectMapper jackson = new ObjectMapper();
		Bid read = jackson.readValue(asString1, Bid.class);
		assertEquals(bid, read);
	}

}
