package geniusweb.issuevalue;

import static org.junit.Assert.assertEquals;

import java.io.IOException;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.LinkedList;
import java.util.List;

import org.junit.Test;

import com.fasterxml.jackson.core.JsonParseException;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.JsonMappingException;
import com.fasterxml.jackson.databind.ObjectMapper;

import geniusweb.issuevalue.NumberValue;
import geniusweb.issuevalue.Value;
import tudelft.utilities.junit.GeneralTests;

public class NumberValueTest extends GeneralTests<NumberValue> {
	private NumberValue value = new NumberValue("12.31");
	private NumberValue value1 = new NumberValue("12.31");
	private NumberValue valueb = new NumberValue("12.310000000000000001");
	private String serialized = "12.31";

	private String bigserialized = "8748703924870439218876954320948372058";
	private NumberValue bigvalue = new NumberValue(bigserialized);

	@Override
	public List<List<NumberValue>> getGeneralTestData() {
		List<List<NumberValue>> list = new LinkedList<>();
		list.add(Arrays.asList(value, value1));
		list.add(Arrays.asList(valueb));
		return list;
	}

	@Override
	public List<String> getGeneralTestStrings() {
		return Arrays.asList("12.31", "12.310000000000000001");
	}

	@Test
	public void testSerialize() throws JsonProcessingException {
		ObjectMapper jackson = new ObjectMapper();
		assertEquals(serialized, jackson.writeValueAsString(value));
	}

	@Test
	public void testDeserialize() throws IOException {
		ObjectMapper jackson = new ObjectMapper();
		assertEquals(value, jackson.readValue(serialized, NumberValue.class));

	}

	@Test
	public void testDeserializeFromValue() throws IOException {
		ObjectMapper jackson = new ObjectMapper();
		assertEquals(value, jackson.readValue(serialized, Value.class));

	}

	@Test
	public void testBigSerialize() throws JsonProcessingException {
		ObjectMapper jackson = new ObjectMapper();
		assertEquals(bigserialized, jackson.writeValueAsString(bigvalue));
	}

	@Test
	public void testBigDeserialize() throws IOException {
		ObjectMapper jackson = new ObjectMapper();
		assertEquals(bigvalue,
				jackson.readValue(bigserialized, NumberValue.class));

	}

	@Test
	public void testSerializeShorts() throws IOException {
		ObjectMapper jackson = new ObjectMapper();
		System.out.println(
				jackson.writeValueAsString(new short[] { 1, 2, 3, 4, 5 }));

	}

	@Test
	public void testDeserializeShorts() throws IOException {
		ObjectMapper jackson = new ObjectMapper();
		ArrayList list = jackson.readValue("[1,2,3,4,5]", ArrayList.class);
		System.out.println(list);
		System.out.println(list.get(0).getClass());

	}

	@Test
	public void testDeserializeMix() throws IOException {
		ObjectMapper jackson = new ObjectMapper();
		ArrayList list = jackson.readValue("[\"short\",1,2,3,4,5]",
				ArrayList.class);
		System.out.println(list);
		System.out.println(list.get(0).getClass());

	}

	/**
	 * Showing that we CAN deserialize big numbers without double quotes
	 * correctly, if we tell jackson upfront that it's a BigDecimal.
	 */
	@Test
	public void testDeserializePlainNumber()
			throws JsonParseException, JsonMappingException, IOException {
		ObjectMapper jackson = new ObjectMapper();
		String valstr = "483958743698732691487326987569213874694328974329874328947320984372498327493827432987231874681273648127";
		BigDecimal val = jackson.readValue(valstr, BigDecimal.class);
		assertEquals(valstr, val.toString());
	}

	@Test
	public void testNumberBlab() throws IOException {
		ObjectMapper jackson = new ObjectMapper();
		// jackson.enable(DeserializationFeature.USE_BIG_DECIMAL_FOR_FLOATS);
		assertEquals(value, jackson.readValue("12.31", Value.class));

	}

}